from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from .validators import text_input_validator, file_input_validator


class UTF8CharField(models.CharField):
    description = _('A char field containing only UTF-8 text')

    def __init__(self, four_byte_detection=False, *args, **kwargs):
        self.four_byte_detection = four_byte_detection
        super(UTF8CharField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(UTF8CharField, self).deconstruct()

        if self.four_byte_detection:
            kwargs['four_byte_detection'] = self.four_byte_detection

        return name, path, args, kwargs

    def validate(self, data, model_instance):
        text_input_validator(data, self.four_byte_detection, ValidationError)
        return super(UTF8CharField, self).validate(data, model_instance)


class UTF8TextField(models.TextField):
    description = _('A text field containing only UTF-8 text')

    def __init__(self, four_byte_detection=False, *args, **kwargs):
        self.four_byte_detection = four_byte_detection
        super(UTF8TextField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(UTF8TextField, self).deconstruct()

        if self.four_byte_detection:
            kwargs['four_byte_detection'] = self.four_byte_detection

        return name, path, args, kwargs

    def validate(self, data, model_instance):
        text_input_validator(data, self.four_byte_detection, ValidationError)
        return super(UTF8TextField, self).validate(data, model_instance)


class UTF8FileField(models.FileField):
    description = _('A text file containing only UTF-8 text')

    def __init__(self, max_content_length=None, four_byte_detection=False, *args, **kwargs):
        self.max_content_length = max_content_length
        self.four_byte_detection = four_byte_detection
        super(UTF8FileField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(UTF8FileField, self).deconstruct()

        if self.max_content_length:
            kwargs['max_content_length'] = self.max_content_length

        if self.four_byte_detection:
            kwargs['four_byte_detection'] = self.four_byte_detection

        return name, path, args, kwargs

    def validate(self, data, model_instance):
        file_input_validator(data, self.max_content_length, self.four_byte_detection, ValidationError)
        return super(UTF8FileField, self).validate(data, model_instance)



try:                        # pragma: no cover
    from rest_framework.serializers import ModelSerializer
    from utf8field.fields import UTF8TextField, UTF8CharField, UTF8FileField
    from utf8field.serializer_fields import UTF8TextSerializerField, UTF8CharSerializerField, UTF8FileSerializerField

    ModelSerializer.serializer_field_mapping[UTF8TextField] = UTF8TextSerializerField
    ModelSerializer.serializer_field_mapping[UTF8CharField] = UTF8CharSerializerField
    ModelSerializer.serializer_field_mapping[UTF8FileField] = UTF8FileSerializerField
except ImportError as e:    # pragma: no cover
    # We won't be needing the restframework anyway
    pass



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from .validators import text_input_validator, file_input_validator


class UTF8TextSerializerField(serializers.CharField):
    def __init__(self, four_byte_detection=False, *args, **kwargs):
        self.four_byte_detection = four_byte_detection
        super(UTF8TextSerializerField, self).__init__(*args, **kwargs)

    def to_internal_value(self, data):
        text_input_validator(data, self.four_byte_detection, ValidationError)
        return data


class UTF8CharSerializerField(serializers.CharField):
    def __init__(self, four_byte_detection=False, *args, **kwargs):
        self.four_byte_detection = four_byte_detection
        super(UTF8CharSerializerField, self).__init__(*args, **kwargs)

    def to_internal_value(self, data):
        text_input_validator(data, self.four_byte_detection, ValidationError)
        return data


class UTF8FileSerializerField(serializers.FileField):
    def __init__(self, max_content_length=None, four_byte_detection=False, *args, **kwargs):
        self.max_content_length = max_content_length
        self.four_byte_detection = four_byte_detection
        super(UTF8FileSerializerField, self).__init__(*args, **kwargs)

    def to_internal_value(self, data):
        file_input_validator(data, self.max_content_length, self.four_byte_detection, ValidationError)
        return data



from __future__ import unicode_literals

import re

from django.utils.translation import ugettext_lazy as _

re_pattern = re.compile(u'[^\u0000-\u07FF\uE000-\uFFFF]', re.UNICODE)


def filter_using_re(unicode_string):
    return re_pattern.sub(u'\uFFFD', unicode_string)


def text_input_validator(data, four_byte_detection, ValidationError):
    if data:
        if '\x00' in data:
            raise ValidationError(
                _('NULL character detected'), code='null')

        if four_byte_detection and data != filter_using_re(data):
            raise ValidationError(
                _('4 Byte UTF8-characters detected'), code='4byte')


def file_input_validator(data, max_content_length, four_byte_detection, ValidationError):
    if data:
        try:
            content = data.read()
            decoded = content.decode('utf-8')

            if four_byte_detection and decoded != filter_using_re(decoded):
                raise ValidationError(
                    _('4 Byte UTF8-characters detected'), code='4byte')

            if max_content_length and len(content) > max_content_length:
                raise ValidationError(_(
                    'The content of the text file cannot be longer then %(max_content_length)s characters.' % {
                        'max_content_length': max_content_length}))

        except UnicodeError:
            raise ValidationError(_('Non UTF8-content detected'), code='utf8')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import sys

from validators import filter_using_re


class Colors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def main():
    try:
        content = read_file(sys.argv[1])
        evaluation = evaluate_characters(content)

        print(get_evaluated_string(evaluation))

    except IndexError:
        print("It's mandatory to supply a path")


def get_evaluated_string(evaluation):
    evaluated_string = ''

    for e in evaluation:
        if e['flagged'] == 'NULL':
            evaluated_string += Colors.WARNING + '[NULL]' + Colors.ENDC
        elif e['flagged'] == '4-BYTE':
            evaluated_string += Colors.OKGREEN + '[' + e['character'] + ']' + Colors.ENDC
        elif e['flagged'] == 'UTF-8':
            evaluated_string += Colors.OKBLUE + '[' + e['character'] + ']' + Colors.ENDC
        else:
            evaluated_string += e['character']

    return evaluated_string


def read_file(path):
    try:
        with io.open(path, mode='r', encoding='utf-8') as fp:
            return fp.read()
    except UnicodeDecodeError:
        try:
            with open(path, mode='r') as fp:
                return fp.read()
        except UnicodeDecodeError:
            print(path + ' seems to be a non UTF-8 file')
            exit()


def evaluate_characters(content):
    evaluation = []

    for i in content:
        character = {
            'character': i,
            'flagged': False
        }

        try:
            d = i

            if d == '\x00':
                character['flagged'] = 'NULL'
            elif filter_using_re(d) != d:
                character['flagged'] = '4-BYTE'
        except UnicodeError:
            character['flagged'] = 'UTF-8'

        evaluation.append(character)
    return evaluation

if __name__ == "__main__":
    main()
